@@ -19,6 +19,7 @@
  */
 
 #include <gtk/gtk.h>
+#include <glib.h>
 #include <fcntl.h>
 
 #include <sys/ioctl.h>
@@ -34,13 +35,19 @@
 #define MAX_LABEL_LENGTH	7
 #define MAX_DEVICE_NAME_LENGTH	32
 #define MAX_COMMAND_LENGTH	512
-#define MAX_PRESET_NAME_LENGTH	15
 
-#define COLOR_SIGNAL_LOW	"#ffff00"
+#define COLOR_SIGNAL_LOW	"#ff0000"
+#define COLOR_SIGNAL_MED	"#ffff00"
 #define COLOR_SIGNAL_HIGH	"#00ff00"
 
 #define BORDER 8
 
+#define DEFAULT_NAME "unnamed"
+#define PRESET_NAMES "preset_names"
+#define PRESET_FREQS "preset_freqs"
+
+const char RADIO_CONFIG[] = "xfce4/panel/radio.rc";
+
 typedef enum {
 	CHANGE_FREQ,
 	CHANGE_PRESET
@@ -48,7 +55,7 @@ typedef enum {
 
 struct radio_preset_st {
 	int			freq;
-	char			name [MAX_PRESET_NAME_LENGTH];
+	gchar			*name;
 	struct radio_preset_st*	next;
 };
 
@@ -66,19 +73,25 @@ typedef struct {
 
 	gboolean		on;
 	gboolean		show_signal;
+	gboolean		auto_update_display;
 	int			freq;
 	int			fd;
 	int			freqfact;
 	char			device [MAX_DEVICE_NAME_LENGTH];
 	char			startup_command [MAX_COMMAND_LENGTH];
 	char			shutdown_command [MAX_COMMAND_LENGTH];
-	radio_preset*		presets;
+	GtkListStore*		presets;
 	mouse_scroll_reaction	scroll;
 
+	// this seems to be card specific and we can't seem to get the maximum
+	// possible value for a given card from v4l
+	double			max_signal_strength;
+
 	XfcePanelPlugin		*plugin;
 } radio_gui;
 
 static void radio_tune (radio_gui*);
-static radio_preset* find_preset_by_freq (int, radio_gui*);
+//static radio_preset* find_preset_by_freq (int, radio_gui*);
 static void read_config (XfcePanelPlugin *, radio_gui *);
-static void write_config (XfcePanelPlugin *, radio_gui *);
+//static void write_config (XfcePanelPlugin *, radio_gui *);
+static void write_config (radio_gui *i, gboolean);
